body {
        font-family:Arial,Helvetica,sans-serif; 
        font-size:11pt;
        color:#000000;
        background-color:#ffffff;
        text-align:left;
}

.bodywidth { 
        width:750px;
}

h1, h2, h3, .contentbar {
        padding:3px;
}


h1, h2, h3 {
        font-weight:bold;
}

h1 { 
        font-size:24pt;
        text-align:center;
}

h2, h3, .contentbar {
        color:#000000;
        background-color:#ccccff;
        border:none;
}

h2 {
        font-size:14pt; 
}

h3 {
        font-size:10pt;
}

img {
        border:0;
}

ul {
        list-style-type:square;
}

pre {
        color:#000000;
        background-color:#cccccc;
        font-family:monospace;
        font-size:8pt;
        padding:3px;
}
